﻿using Microsoft.ServiceBus.Messaging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MyLogicApp.AcceptanceTests.Utilities;
using MyLogicApp.AcceptanceTests.Utilities.LogicAppDiagnostics;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using TechTalk.SpecFlow;

namespace MyLogicApp.AcceptanceTests.Features.Whitebox
{
    [Binding]
    [Scope(Feature = "Whitebox Test")]
    public class Steps
    {
        [Given(@"the strorage account is empty")]
        public void GivenTheStrorageAccountIsEmpty(Table t)
        {
            var connection = ConfigurationManager.ConnectionStrings["ArchiveStorageConnectionString"].ConnectionString;
            var storageHelper = new Utilities.AzureBlobStorageHelper(connection);

            foreach (var row in t.Rows)
            {
                var containerName = row[0];
                storageHelper.EmptyContainer(containerName);
                Assert.IsTrue(storageHelper.Exists(containerName), string.Format("Storage Container {0} doesnt exist", containerName));
            }
        }

        [Given(@"the queue is empty")]
        public void GivenTheQueueIsEmpty()
        {
            //We can assume queue is empty
        }

        [Given(@"the logic app is enabled")]
        public void GivenTheLogicAppIsEnabled()
        {
            //ScenarioContext.Current.Pending();
        }

        [Given(@"the diagnostics tracking manager is started")]
        public void GivenTheDiagnosticsTrackingManagerIsStarted()
        {
            TrackingManager.Start();
        }


        [When(@"a message is submitted to the queue")]
        public void WhenAMessageIsSubmittedToTheQueue(Table t)
        {
            var label = t.Rows[0][0];
            var connectionString = ConfigurationManager.ConnectionStrings["ServiceBus"].ConnectionString;

            var messageBodyStream = new MemoryStream();
            var sw = new StreamWriter(messageBodyStream);
            sw.Write("this is a test");
            sw.Flush();
            messageBodyStream.Flush();
            messageBodyStream.Seek(0, SeekOrigin.Begin);

            var message = new BrokeredMessage(messageBodyStream, false);
            message.Label = label;  //Label used on condition

            var queueClient = QueueClient.CreateFromConnectionString(connectionString, "logicapps-input");
            queueClient.Send(message);
        }

        [Then(@"the logic app will receive the message")]
        public void ThenTheLogicAppWillReceiveTheMessage()
        {
            var eventMessage = "When_a_message_is_received_in_a_queue_(auto-complete)";
            var found = TrackingManager.ContainsRunningEventForResource(eventMessage);
            Assert.IsTrue(found, "Could not find the start trigger");
        }

        [Then(@"the logic app will decide which route to process the message")]
        public void ThenTheLogicAppWillDecideWhichRouteToProcessTheMessage()
        {
            var eventMessage = "Condition_-_Which_Blob_Container";
            var found = TrackingManager.ContainsSuccessEventForResource(eventMessage);
            Assert.IsTrue(found, "Could not find the event for the condition executing");
        }

        [Then(@"the logic app will flow the message to the yes folder")]
        public void ThenTheLogicAppWillFlowTheMessageToTheYesFolder()
        {
            var eventMessage = "Create_blob_-_YES";
            var found = TrackingManager.ContainsSuccessEventForResource(eventMessage);
            Assert.IsTrue(found, "Could not find the event for blob create in yes folder");
        }

        [Then(@"the message will be saved into the storage location")]
        public void ThenTheMessageWillBeSavedIntoTheStorageLocation(Table table)
        {
            var connection = ConfigurationManager.ConnectionStrings["ArchiveStorageConnectionString"].ConnectionString;
            var storageHelper = new Utilities.AzureBlobStorageHelper(connection);
            var blobList = storageHelper.GetBlobList(table.Rows[0][0]);
            Assert.IsTrue(blobList.Count == 1, "There should be only 1 blob");
        }

        [Then(@"the logic app will flow the message to the default folder")]
        public void ThenTheLogicAppWillFlowTheMessageToTheDefaultFolder()
        {
            var eventMessage = "Create_blob_-_Default";
            var found = TrackingManager.ContainsSuccessEventForResource(eventMessage);
            Assert.IsTrue(found, "Could not find the event for blob create in default folder");
        }

    }
}
