﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MyLogicApp.AcceptanceTests.Utilities.LogicAppDiagnostics
{
    public class LogicAppTrackingEvent
    {
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("OperationName: " + OperationName);
            sb.Append("ResourceID: " + ResourceID);
            sb.Append("Time: " + Time);
            sb.Append("StatusDescription: " + StatusDescription);            

            return sb.ToString();
        }
        public Record RecordObject { get; set; }

        public string OperationName { get; set; }
        public string ResourceID { get; set; }
        public string Time { get; set; }

        public string StatusDescription { get; set; }

        public bool Succeeded { get; set; }

        public bool Running { get; set; }

        public bool Failed { get; set; }

        public static List<LogicAppTrackingEvent> Convert(string text)
        {
            var logicAppEvents = new List<LogicAppTrackingEvent>();

            var rootObject = JsonConvert.DeserializeObject<RootObject>(text);
            foreach(var record in rootObject.records)
            {
                var operationName = record.operationName;
                var resourceId = record.resourceId;
                var dt = record.time;
                
                var trackingEvent = new LogicAppTrackingEvent();
                trackingEvent.RecordObject = record;
                trackingEvent.ResourceID = resourceId;
                trackingEvent.Time = dt;
                trackingEvent.OperationName = operationName;
                trackingEvent.StatusDescription = record.properties.status;

                if(record.properties.status.ToLower() == "succeeded")
                {
                    trackingEvent.Succeeded = true;
                }
                else if (record.properties.status.ToLower() == "failed")
                {
                    trackingEvent.Failed = true;
                }
                else if (record.properties.status.ToLower() == "running")
                {
                    trackingEvent.Running = true;
                }

                Console.WriteLine(trackingEvent.ToString());
                Console.WriteLine();
                logicAppEvents.Add(trackingEvent);
            }

            return logicAppEvents;
        }
        
    }


    public class Tags
    {
        public string displayName { get; set; }
    }

    public class Resource
    {
        public string subscriptionId { get; set; }
        public string resourceGroupName { get; set; }
        public string workflowId { get; set; }
        public string workflowName { get; set; }
        public string runId { get; set; }
        public string location { get; set; }
    }

    public class Correlation
    {
        public string clientTrackingId { get; set; }
    }

    public class Properties
    {
    public string startTime { get; set; }
    public string endTime { get; set; }
    public string status { get; set; }
    public string code { get; set; }
    public Tags tags { get; set; }
    public Resource resource { get; set; }
    public Correlation correlation { get; set; }
}

    public class Record
{
    public string time { get; set; }
    public string workflowId { get; set; }
    public string resourceId { get; set; }
    public string category { get; set; }
    public string level { get; set; }
    public string operationName { get; set; }
    public Properties properties { get; set; }
}

    public class RootObject
{
    public List<Record> records { get; set; }
}

}
