﻿using Microsoft.ServiceBus.Messaging;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace MyLogicApp.AcceptanceTests.Utilities.LogicAppDiagnostics
{
    public class TrackingManager
    {
        private static DateTime _startDateTime;
        private static EventProcessorHost _eventProcessorHost;
        public static List<LogicAppTrackingEvent> LogicAppEvents = new List<LogicAppTrackingEvent>();

        public static void AddLogicAppEvent(LogicAppTrackingEvent logicAppEvent)
        {
            var eventDate = DateTime.Parse(logicAppEvent.Time);

            if (eventDate < _startDateTime)
                return; // Ignore event as its old

            LogicAppEvents.Add(logicAppEvent);
        }

        public static bool ContainsSuccessEventForResource(string text, int timeoutSeconds = 120)
        {
            var timeout = DateTime.Now.AddSeconds(timeoutSeconds);

            while (DateTime.Now < timeout)
            {
                foreach (var logicAppEvent in LogicAppEvents)
                {
                    if (logicAppEvent.ResourceID.ToUpper().Contains(text.ToUpper()))
                        if (logicAppEvent.Succeeded)
                            return true;
                }

                Thread.Sleep(new TimeSpan(0, 0, 2));
            }
            return false;
        }

        public static bool ContainsRunningEventForResource(string text, int timeoutSeconds = 120)
        {
            var timeout = DateTime.Now.AddSeconds(timeoutSeconds);

            while (DateTime.Now < timeout)
            {
                foreach (var logicAppEvent in LogicAppEvents)
                {
                    if (logicAppEvent.ResourceID.ToUpper().Contains(text.ToUpper()))
                        if (logicAppEvent.Running)
                            return true;
                }

                Thread.Sleep(new TimeSpan(0, 0, 2));
            }
            return false;
        }


        public static void Start()
        {
            _startDateTime = DateTime.UtcNow;
            string eventProcessorHostName = Guid.NewGuid().ToString();
            _eventProcessorHost = new EventProcessorHost(eventProcessorHostName, DiagnosticsConfig.EventHubName, EventHubConsumerGroup.DefaultGroupName, DiagnosticsConfig.EventHubConnectionString, DiagnosticsConfig.StorageConnectionString);

            Console.WriteLine("Registering EventProcessor...");
            var options = new EventProcessorOptions()
            {
                InitialOffsetProvider = (partitionid) => DateTime.UtcNow
            };
            options.ExceptionReceived += (sender, e) => { Console.WriteLine(e.Exception); };
            options.PrefetchCount = 10;
            _eventProcessorHost.RegisterEventProcessorAsync<SimpleEventProcessor>(options).Wait();            
        }
        public static void Clear()
        {
            LogicAppEvents = new List<LogicAppTrackingEvent>();            
        }
        public static void Stop()
        {
            _eventProcessorHost.UnregisterEventProcessorAsync().Wait();

            Console.WriteLine();
            Console.WriteLine("LogicApp Events");
            foreach(var evt in LogicAppEvents)
            {
                Console.WriteLine(evt.OperationName + " - " + evt.ResourceID);
            }
            Console.WriteLine();
            Console.WriteLine();
            LogicAppEvents = new List<LogicAppTrackingEvent>();
        }
    }

}
